import { createHistory as createHistory$1 } from '@gact/history';
import React, { forwardRef, useMemo, useCallback, useEffect } from 'react';

/**
 * Creates a `GactHistory`.
 *
 * @remarks
 * Enforces that history state is compatible with the Gact store.
 */
function createHistory() {
    return createHistory$1();
}

function isModifiedEvent(event) {
    return event.metaKey || event.altKey || event.ctrlKey || event.shiftKey;
}
/**
 * Creates a `Link` component.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createLink({ useValue, path, }) {
    return function (locationPath, history) {
        return forwardRef(function Link({ children, href, state, className, style = {}, activeClassName, activeStyle = {}, replace = false, onClick, ...rest }, ref) {
            const pathname = useMemo(() => new URL(href, href.startsWith("/") ? window.location.origin : undefined).pathname, [href]);
            const isActive = useValue(path(locationPath, "pathname"), (currentPathname) => currentPathname === pathname);
            const finalClassName = useMemo(() => isActive
                ? [className, activeClassName].filter((c) => c).join(" ")
                : className, [className, activeClassName, isActive]);
            const finalStyle = isActive ? { ...style, ...activeStyle } : style;
            const navigate = useCallback(function () {
                const method = replace ? history.replace : history.push;
                method(href, state);
            }, [href, replace, state]);
            function handleClick(event) {
                try {
                    if (onClick) {
                        onClick(event);
                    }
                }
                catch (err) {
                    // ensure we don't navigate away on error from provided onClick
                    event.preventDefault();
                    throw err;
                }
                if (!event.defaultPrevented && // onClick prevented default
                    event.button === 0 && // ignore everything but left clicks
                    (!rest.target || rest.target === "_self") && // let browser handle "target=_blank" etc.
                    !isModifiedEvent(event) // ignore clicks with modifier keys
                ) {
                    event.preventDefault();
                    navigate();
                }
            }
            return (React.createElement("a", Object.assign({ ref: ref, href: href, onClick: handleClick, className: finalClassName, style: finalStyle }, rest), children));
        });
    };
}

function createRedirect(history) {
    return function Redirect({ url, state }) {
        useEffect(() => {
            history.push(url, state);
        }, [url]);
        return null;
    };
}

/**
 * Decomposes a path into segments.
 */
function segmentize(path) {
    let start = 0;
    let end = path.length;
    if (path.startsWith("/")) {
        start++;
    }
    if (path.endsWith("/")) {
        end--;
    }
    return path.slice(start, end).split("/");
}

/**
 * Determines if the provided pathname matches the provided path.
 */
function doesPathMatch(pathname, path) {
    const paths = Array.isArray(path) ? path : [path];
    const allPathnameParts = segmentize(pathname);
    for (const path of paths) {
        const pathParts = segmentize(path);
        let pathnameParts = allPathnameParts;
        if (pathParts[pathParts.length - 1] == "**") {
            pathParts.pop();
            pathnameParts = allPathnameParts.slice(0, pathParts.length);
        }
        // match only possible for paths of the same length
        if (pathParts.length !== pathnameParts.length) {
            continue;
        }
        if (pathParts.every((part, i) => (part === "*" && pathnameParts[i].length > 0) ||
            part === pathnameParts[i])) {
            return true;
        }
    }
    return false;
}

/**
 * Creates a `Route` component.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createRoute({ useValue }) {
    return function (locationPath) {
        return function Route({ children, path, switchMatch, }) {
            const location = useValue(locationPath);
            if (switchMatch || doesPathMatch(location.pathname, path)) {
                return children;
            }
            return null;
        };
    };
}

var SegmentPoints;
(function (SegmentPoints) {
    SegmentPoints[SegmentPoints["Root"] = 1] = "Root";
    SegmentPoints[SegmentPoints["Wildcard"] = -1] = "Wildcard";
    SegmentPoints[SegmentPoints["Dynamic"] = 2] = "Dynamic";
    SegmentPoints[SegmentPoints["Static"] = 3] = "Static";
    SegmentPoints[SegmentPoints["Any"] = 4] = "Any";
})(SegmentPoints || (SegmentPoints = {}));
/**
 * Ranks the provided path.
 *
 * @remarks
 * - The rank is used by <Switch /> to select a <Route />.
 *
 * - The rank algorithm is borrowed from the {@link https://github.com/reach/router | Reach Router}.
 */
function rankPath(path) {
    let score = 0;
    for (const segment of segmentize(path)) {
        score += SegmentPoints.Any;
        if (segment === "") {
            score += SegmentPoints.Root;
        }
        else if (segment === "*") {
            score += SegmentPoints.Dynamic;
        }
        else if (segment === "**") {
            score += SegmentPoints.Wildcard;
        }
        else {
            score += SegmentPoints.Static;
        }
    }
    return score;
}

/**
 * Creates a `Switch` component.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createSwitch({ path, useValue, }) {
    return function (locationPath, Route) {
        return function Switch({ children }) {
            const pathname = useValue(path(locationPath, "pathname"));
            const match = React.Children.toArray(children)
                .reduce(function (routes, child, index) {
                if (!(React.isValidElement(child) && child.type === Route)) {
                    throw Error("Only <Route /> is allowed to be a child of <Switch />");
                }
                const { path: pathProp } = child.props;
                const paths = Array.isArray(pathProp) ? pathProp : [pathProp];
                for (const path of paths) {
                    routes.push({ child, index, path, rank: rankPath(path) });
                }
                return routes;
            }, [])
                .sort((a, b) => a.rank < b.rank ? 1 : a.rank > b.rank ? -1 : a.index - b.index)
                .find(({ path }) => doesPathMatch(pathname, path));
            if (match) {
                return React.cloneElement(match.child, { switchMatch: true });
            }
            else {
                return null;
            }
        };
    };
}

/**
 * Extracts path parameters from the provided pathname if the pathname matches
 * the path.
 *
 * The path is like a regular path expect that wildcards `*` are replaced by `:<parameterIdentifer>`
 */
function computePathParams(pathname, path) {
    const pathSegments = segmentize(path);
    const pathToMatch = "/" +
        pathSegments
            .map((segment) => (segment.startsWith(":") ? "*" : segment))
            .join("/");
    const params = {};
    if (doesPathMatch(pathname, pathToMatch)) {
        const pathnameSegments = segmentize(pathname);
        for (const [index, segment] of pathSegments.entries()) {
            if (segment.startsWith(":")) {
                params[segment.slice(1)] = pathnameSegments[index];
            }
        }
    }
    else {
        for (const segment of pathSegments) {
            if (segment.startsWith(":")) {
                params[segment.slice(1)] = null;
            }
        }
    }
    return params;
}

/**
 * Creates a `usePathParams` hook.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createUsePathParams({ useValue, }) {
    return function (locationPath) {
        return function UsePathParams(path) {
            const location = useValue(locationPath);
            return computePathParams(location.pathname, path);
        };
    };
}

/**
 * Creates a `useSearchParams` hook.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createUseSearchParams({ path, useValue, }) {
    return function (locationPath) {
        return function useSearchParams() {
            return new URLSearchParams(useValue(path(locationPath, "search")));
        };
    };
}

/**
 * Creates a `Router`.
 *
 * @typeParam S - the Gact store state
 * @typeParam HS - the history state
 */
function createRouter(store) {
    return function (locationPath, history) {
        /**
         * Sync location with store state.
         * No need to cleanup since this will happen for the app's lifetime.
         */
        history.subscribe(function (location) {
            store.set(locationPath, location);
        });
        const Route = createRoute(store)(locationPath);
        const Switch = createSwitch(store)(locationPath, Route);
        const Link = createLink(store)(locationPath, history);
        const Redirect = createRedirect(history);
        const useSearchParams = createUseSearchParams(store)(locationPath);
        const usePathParams = createUsePathParams(store)(locationPath);
        return { Link, Redirect, Route, Switch, usePathParams, useSearchParams };
    };
}

export { createHistory, createRouter, doesPathMatch };
//# sourceMappingURL=react-router.es5.js.map
